package it.openutils.web.jstl;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.jstl.core.Config;
import javax.servlet.jsp.jstl.fmt.LocalizationContext;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author fgiust
 * @version $Revision: 148 $ ($Author: fgiust $)
 */
public class ElxFunctions
{

    /**
     * Logger.
     */
    private static Logger log = LoggerFactory.getLogger(ElxFunctions.class);

    /**
     * check is a collection is empty.
     * @param collection
     * @return boolean
     */
    public static boolean isCollectionEmpty(Object collection)
    {

        if (collection == null)
        {
            return true;
        }

        try
        {
            if (collection instanceof Collection)
            {
                return ((Collection) collection).isEmpty();
            }
            else if (collection instanceof Map)
            {
                return ((Map) collection).isEmpty();
            }
        }
        catch (Throwable e)
        {
            // catch hibernate lazy initialization errors
            log.error("An error occurred while evaluating the elx:isempty function: " + e.getMessage(), e);
            return true;
        }

        // the "new Throwable()" is used to add a stacktrace
        log
            .error(
                "elx:isempty can be used only with an object of type java.util.Collection or java.util.Map, you are trying to use it with an instance of "
                    + collection.getClass(),
                new Throwable());

        return true;

    }

    /**
     * return the size of the collection.
     * @param collection
     * @return int
     */
    public static int getCollectionSize(Object collection)
    {

        if (collection == null)
        {
            return 0;
        }

        try
        {
            if (collection instanceof Collection)
            {
                return ((Collection) collection).size();
            }
            else if (collection instanceof Map)
            {
                return ((Map) collection).size();
            }
        }
        catch (Throwable e)
        {
            // catch hibernate lazy initialization errors
            log.error("An error occurred while evaluating the elx:size function: " + e.getMessage(), e);
            return 0;
        }

        // the "new Throwable()" is used to add a stacktrace
        log
            .error(
                "elx:size can be used only with an object of type java.util.Collection or java.util.Map, you are trying to use it with an instance of "
                    + collection.getClass(),
                new Throwable());
        return 0;
    }

    public static String formatMessage(PageContext pageContext, String key)
    {
        String title = null;
        ResourceBundle bundle = null;

        // standard jstl localizationContest
        // check for the localizationContext in applicationScope, set in web.xml
        LocalizationContext localization = getLocalizationContext(pageContext);

        if (localization != null)
        {
            bundle = localization.getResourceBundle();
        }

        if (bundle != null)
        {
            try
            {
                title = bundle.getString(key);
            }
            catch (MissingResourceException e)
            {
                // ignore
            }
        }

        return title;
    }

    /**
     * Check if a given array of Strings contains <em>the String representation</em> of an object.
     * @param array String array
     * @param value object to search for
     * @return <code>true</code> if a array contains a value equals to the String representation of object
     */
    public static boolean containsString(String[] array, Object value)
    {
        if (log.isDebugEnabled())
        {
            // log.debug("containsString [" + value + "] in [" + ArrayUtils.toString(array) + "]");
        }

        if (array == null || value == null)
        {
            return false;
        }

        String valueString = value.toString();

        for (String element : array)
        {
            if (valueString.equals(element))
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Compares the String representation of two objects.
     * @param one first object to be compared
     * @param two second object to be compared
     * @return <code>true</code> only if both objects are not null and their String representaion is equals()
     */
    public static boolean equalsString(Object one, Object two)
    {
        if (one == null || two == null)
        {
            return false;
        }

        if (log.isDebugEnabled())
        {
            log.debug("Comparing [" + one + "] and [" + two + "]");
        }

        return one.toString().equals(two.toString());
    }

    /**
     * Returns a bean property (optionally nested= handling nulls both in bean and property name)
     * @param bean bean
     * @param property property name, optionally nested
     * @return property value, null if bean is null, bean if property is null
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public static Object getProperty(Object bean, String property)
    {
        if (bean == null)
        {
            return null;
        }
        if (StringUtils.isEmpty(property))
        {
            return bean;
        }

        try
        {
            if (property.startsWith("(string)"))
            {
                return StringUtils.substringAfter(property, "(string)");
            }
            else
            {
                return PropertyUtils.getProperty(bean, property);
            }

        }
        catch (Exception e)
        {
            log.error("Error while evaluating property \"" + property + "\" in bean " + bean.getClass().getName(), e);
        }

        return StringUtils.EMPTY;
    }

    /**
     * Check if ricambio is into the inverval of tavola verifying dates
     * @param tavolaDa start date for tavola
     * @param tavolaA end date for tavola
     * @param ricambioDa start date for ricambio
     * @param ricambioA end date for ricambio
     * @return
     */
    public static boolean dateRange(java.util.Date tavolaDa, java.util.Date tavolaA, java.util.Date ricambioDa,
        java.util.Date ricambioA)
    {
        long tavDa = (tavolaDa == null) ? 0 : tavolaDa.getTime();
        long tavA = (tavolaA == null) ? Long.MAX_VALUE : tavolaA.getTime();

        long ricDa = (ricambioDa == null) ? 0 : ricambioDa.getTime();
        long ricA = (ricambioA == null) ? Long.MAX_VALUE : ricambioA.getTime();

        return (ricDa <= tavA && ricA >= tavDa);
    }

    public static String space()
    {
        return " ";
    }

    /**
     * Gets the default I18N localization context.
     * @param pc Page in which to look up the default I18N localization context
     */
    private static LocalizationContext getLocalizationContext(PageContext pc)
    {
        LocalizationContext locCtxt = null;

        Object obj = Config.find(pc, Config.FMT_LOCALIZATION_CONTEXT);
        if (obj == null)
        {
            return null;
        }

        if (obj instanceof LocalizationContext)
        {
            locCtxt = (LocalizationContext) obj;
        }
        else
        {
            // localization context is a bundle basename
            locCtxt = getLocalizationContext(pc, (String) obj);
        }

        return locCtxt;
    }

    private static LocalizationContext getLocalizationContext(PageContext pc, String basename)
    {
        LocalizationContext locCtxt = null;
        ResourceBundle bundle = null;

        if ((basename == null) || basename.equals(""))
        {
            return new LocalizationContext();
        }

        return locCtxt;
    }

}
